<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerReferralPlugin\Application\EventListener;

use Dedi\SyliusCustomerReferralPlugin\Entity\InviteeInterface;
use Sylius\Component\Core\Model\OrderInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\EventDispatcher\GenericEvent;

class CustomerOrderPostCompleteListener
{
    /** @var EventDispatcherInterface */
    private $eventDispatcher;

    public function __construct(
        EventDispatcherInterface $eventDispatcher
    ) {
        $this->eventDispatcher = $eventDispatcher;
    }

    public function __invoke(GenericEvent $event): void
    {
        /** @var OrderInterface $order */
        $order = $event->getSubject();

        /** @var InviteeInterface $invitee */
        $invitee = $order->getUser();
        if (!$referral = $invitee->getReferral()) {
            return;
        }

        $this->eventDispatcher->dispatch(
            new GenericEvent($referral->getInviter()->getCustomer()),
            'dedi_sylius_customer_referral_plugin.inviter_has_invitee_order_complete'
        );
    }
}
