<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerReferralPlugin\Infrastructure\Context;

use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Model\ReferralModelInterface;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Repository\ReferralRepositoryInterface;
use Symfony\Component\HttpFoundation\RequestStack;

class RegisterReferralContext
{
    public const REGISTER_REFERRAL_QUERY_KEY = 'referralCode';

    /** @var ReferralModelInterface|null */
    private $referral;

    public function __construct(
        RequestStack $requestStack,
        ReferralRepositoryInterface $referralRepository
    ) {
        $request = $requestStack->getMasterRequest();

        if (!$request) {
            return;
        }

        if (!$code = $this->getCode($request)) {
            return;
        }

        $this->referral = $referralRepository->findByCode($code);
    }

    public function getReferral(): ?ReferralModelInterface
    {
        return $this->referral;
    }

    /**
     * Get code either from post data or query params
     */
    private function getCode($request): ?string
    {
        $formData = $request->get('sylius_customer_registration');
        if ($formData && array_key_exists('code', $formData)) {
            return $formData['code'];
        }

        return $request->query->get(self::REGISTER_REFERRAL_QUERY_KEY);
    }
}
