<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerReferralPlugin\Infrastructure\Repository;

use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Exception\InviteeAlreadyInvitedException;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Model\ReferralModelInterface;
use Dedi\SyliusCustomerReferralPlugin\Domain\Referral\Repository\ReferralRepositoryInterface;
use Sylius\Bundle\ResourceBundle\Doctrine\ORM\EntityRepository;

class ReferralRepository extends EntityRepository implements ReferralRepositoryInterface
{
    public function save(ReferralModelInterface $referralModel): ReferralModelInterface
    {
        // do not rely on the entity manager to throw a "unique constraint violation exception"
        // in order to know if an email has already been invited because it will close the entity manager
        // https://stackoverflow.com/questions/44778618/entitymanager-is-closed-due-unique-violation
        if ($this->findOneBy(['inviteeEmail' => $referralModel->getInviteeEmail()])) {
            throw new InviteeAlreadyInvitedException();
        }

        $this->add($referralModel);

        return $referralModel;
    }

    public function findByInviteeEmail(?string $inviteeEmail): ?ReferralModelInterface
    {
        /** @var ReferralModelInterface $referral */
        $referral = $this->findOneBy([
            'inviteeEmail' => $inviteeEmail,
        ]);

        return $referral;
    }

    public function findByCode(string $code): ?ReferralModelInterface
    {
        [$id, $inviteeEmail] = explode(':', base64_decode($code));

        /** @var ReferralModelInterface $referral */
        $referral = $this->findOneBy([
            'id' => $id,
            'inviteeEmail' => $inviteeEmail,
        ]);

        return $referral;
    }
}
